==> ioiocore-2.3.1-cp38-cp38-macosx_11_0_universal2.whl: ioiocore/io_node.py <==
from typing import Any
from copy import deepcopy
from .i_node import INode
from .o_node import ONode
from .i_node import IPort
from .o_node import OPort
from .constants import Constants

import ioiocore.imp as imp


class IONode(INode, ONode):
    """
    A class representing a node with input and output ports, inheriting from
    both INode and ONode.
    """

    class Configuration(INode.Configuration, ONode.Configuration):
        """
        Configuration class for IONode.
        """

        class Keys(INode.Configuration.Keys, ONode.Configuration.Keys):
            """
            Keys for the IONode configuration (none except the inherited ones).
            """
            DECIMATION_FACTOR = "decimation_factor"

        def __init__(self,
                     input_ports: list = None,
                     output_ports: list = None,
                     decimation_factor: int = None,
                     **kwargs):
            """
            Initializes the configuration for IONode.

            Parameters
            ----------
            input_ports : list of IPort.Configuration, optional
                A list of input port configurations (default is None).
            output_ports : list of OPort.Configuration, optional
                A list of output port configurations (default is None).
            decimation_factor : factor by which the output data is decimated
                (default is 1).
            **kwargs : additional keyword arguments
                Other configuration options.
            """
            if input_ports is None:
                input_ports = [IPort.Configuration()]
            if output_ports is None:
                output_ports = [OPort.Configuration()]
            if decimation_factor is None:
                decimation_factor = 1

            INode.Configuration.__init__(self,
                                         input_ports=input_ports,
                                         output_ports=output_ports,
                                         decimation_factor=decimation_factor,
                                         **kwargs)
            ONode.Configuration.__init__(self,
                                         input_ports=input_ports,
                                         output_ports=output_ports,
                                         decimation_factor=decimation_factor,
                                         **kwargs)

    _IMP_CLASS = imp.IONodeImp
    _imp: _IMP_CLASS  # for type hinting  # type: ignore
    config: Configuration  # for type hinting

    def __init__(self,
                 input_ports: list = None,
                 output_ports: list = None,
                 decimation_factor: int = None,
                 **kwargs):
        """
        Initializes the IONode.

        Parameters
        ----------
        input_ports : list of IPort.Configuration, optional
            A list of input port configurations (default is None).
        output_ports : list of OPort.Configuration, optional
            A list of output port configurations (default is None).
        decimation_factor : factor by which the output data is decimated
            (default is 1).
        **kwargs : additional keyword arguments
            Other configuration options.
        """
        self.create_config(input_ports=input_ports,
                           output_ports=output_ports,
                           decimation_factor=decimation_factor,
                           **kwargs)
        self.create_implementation()
        super().__init__(**self.config)

    def setup(self,
              data: dict,
              port_metadata_in: dict) -> dict:
        """
        Standard implementation of the setup method. Only allowed for
        one input port. If subclasses have more than one input port, they
        must overload this method.

        Parameters
        ----------
        data : dict
            A dictionary containing setup data.
        port_metadata_in : dict
            A dictionary containing input port metadata.

        Returns
        -------
        dict
            A dictionary containing output port metadata.

        Raises
        ------
        ValueError
            If the number of input ports is not exactly one.
        """
        if len(port_metadata_in) != 1:
            raise ValueError("Default implementation of setup() requires "
                             "exactly one input port. Please overload this "
                             "method appropriately.")
        port_metadata_out: dict = {}
        ip_config = self.config[self.config.Keys.INPUT_PORTS]
        ip_names = [s[self.Configuration.Keys.NAME] for s in ip_config]
        op_config = self.config[self.config.Keys.OUTPUT_PORTS]
        op_names = [s[self.Configuration.Keys.NAME] for s in op_config]
        M = self.config[self.config.Keys.DECIMATION_FACTOR]
        for ip_name in ip_names:
            for op_name in op_names:
                md = deepcopy(port_metadata_in[ip_name])
                if Constants.Keys.SAMPLING_RATE in md.keys():
                    md[Constants.Keys.SAMPLING_RATE] /= M
                port_metadata_out[op_name] = md
        return port_metadata_out

    def _is_output_step(self) -> bool:
        """
        Indicates if an output sample is generated at the current
        execution step. Always true if decimation factor M == 1. True
        only every Mth sample if M > 1.

        Returns
        -------
        bool
            Logical value indicating if an output sample is generated.

        """
        return self._imp.is_output_step()

==> ioiocore-4.0.1-cp38-cp38-macosx_11_0_universal2.whl: ioiocore/io_node.py <==
from typing import Any
from copy import deepcopy
from .i_node import INode
from .o_node import ONode
from .i_node import IPort
from .o_node import OPort
from .constants import Constants

import ioiocore.imp as imp


class IONode(INode, ONode):
    """
    A class representing a node with input and output ports, inheriting from
    both INode and ONode.
    """

    class Configuration(INode.Configuration, ONode.Configuration):
        """
        Configuration class for IONode.
        """

        class Keys(INode.Configuration.Keys, ONode.Configuration.Keys):
            """
            Keys for the IONode configuration (none except the inherited ones).
            """
            pass

        def __init__(self,
                     input_ports: list = None,
                     output_ports: list = None,
                     **kwargs):
            """
            Initializes the configuration for IONode.

            Parameters
            ----------
            input_ports : list of IPort.Configuration, optional
                A list of input port configurations (default is None).
            output_ports : list of OPort.Configuration, optional
                A list of output port configurations (default is None).
            **kwargs : additional keyword arguments
                Other configuration options.
            """
            if input_ports is None:
                input_ports = [IPort.Configuration()]
            if output_ports is None:
                output_ports = [OPort.Configuration()]

            INode.Configuration.__init__(self,
                                         input_ports=input_ports,
                                         output_ports=output_ports,
                                         **kwargs)
            ONode.Configuration.__init__(self,
                                         input_ports=input_ports,
                                         output_ports=output_ports,
                                         **kwargs)

    _IMP_CLASS = imp.IONodeImp
    _imp: _IMP_CLASS  # for type hinting  # type: ignore
    config: Configuration  # for type hinting

    def __init__(self,
                 input_ports: list = None,
                 output_ports: list = None,
                 **kwargs):
        """
        Initializes the IONode.

        Parameters
        ----------
        input_ports : list of IPort.Configuration, optional
            A list of input port configurations (default is None).
        output_ports : list of OPort.Configuration, optional
            A list of output port configurations (default is None).
        decimation_factor : factor by which the output data is decimated
            (default is 1).
        **kwargs : additional keyword arguments
            Other configuration options.
        """
        self.create_config(input_ports=input_ports,
                           output_ports=output_ports,
                           **kwargs)
        self.create_implementation()
        super().__init__(**self.config)

    def setup(self,
              data: dict,
              port_context_in: dict) -> dict:
        """
        Standard implementation of the setup method. Only allowed for
        one input port. If subclasses have more than one input port, they
        must overload this method.

        Parameters
        ----------
        data : dict
            A dictionary containing setup data.
        port_context_in : dict
            A dictionary containing input port context.

        Returns
        -------
        dict
            A dictionary containing output port context.

        Raises
        ------
        ValueError
            If the number of input ports is not exactly one.
        """
        if len(port_context_in) != 1:
            raise ValueError("Default implementation of setup() requires "
                             "exactly one input port. Please overload this "
                             "method appropriately.")
        port_context_out: dict = {}
        ip_config = self.config[self.config.Keys.INPUT_PORTS]
        ip_names = [s[self.Configuration.Keys.NAME] for s in ip_config]
        op_config = self.config[self.config.Keys.OUTPUT_PORTS]
        op_names = [s[self.Configuration.Keys.NAME] for s in op_config]
        for ip_name in ip_names:
            for op_name in op_names:
                md = deepcopy(port_context_in[ip_name])
                port_context_out[op_name] = md
        return port_context_out
